"""byte-pysqlite - executor package."""
from __future__ import absolute_import, division, print_function

from byte.executors.pysqlite.main import PySqliteExecutor

__all__ = (
    'PySqliteExecutor',
)
